@@ -2,7 +2,7 @@ package TheSchwartz::Moosified::Job;
 
 use Moose;
 use Storable ();
-use TheSchwartz::Moosified::Utils qw/sql_for_unixtime run_in_txn/;
+use TheSchwartz::Moosified::Utils qw/sql_for_unixtime/;
 use TheSchwartz::Moosified::JobHandle;
 
 has 'jobid'         => ( is => 'rw', isa => 'Int' );
@@ -10,7 +10,7 @@ has 'funcid'        => ( is => 'rw', isa => 'Int' );
 has 'arg'           => ( is => 'rw', isa => 'Any' );
 has 'uniqkey'       => ( is => 'rw', isa => 'Maybe[Str]' );
 has 'insert_time'   => ( is => 'rw', isa => 'Maybe[Int]' );
-has 'run_after'     => ( is => 'rw', isa => 'Int', default => sub { time } );
+has 'run_after'     => ( is => 'rw', isa => 'Int', default => time() );
 has 'grabbed_until' => ( is => 'rw', isa => 'Int', default => 0 );
 has 'priority'      => ( is => 'rw', isa => 'Maybe[Int]' );
 has 'coalesce'      => ( is => 'rw', isa => 'Maybe[Str]' );
@@ -18,20 +18,19 @@ has 'coalesce'      => ( is => 'rw', isa => 'Maybe[Str]' );
 has 'funcname' => (
     is => 'rw',
     isa => 'Str',
-    lazy_build => 1,
-);
-
-has 'handle' => (
-    is => 'rw',
-    isa => 'TheSchwartz::Moosified::JobHandle',
-    handles => [qw(
-        exit_status
-        failure_log
-        failures
-        client
-        dbh
-    )],
+    lazy => 1,
+    default => sub {
+        my ($self) = @_;
+        
+        my $handle = $self->handle;
+        my $client = $handle->client;
+        my $dbh    = $handle->dbh;
+        my $funcname = $client->funcid_to_name($dbh, $self->funcid)
+            or die "Failed to lookup funcname of job $self";
+        return $funcname;
+    }
 );
+has 'handle' => ( is => 'rw', isa => 'TheSchwartz::Moosified::JobHandle' );
 
 has 'did_something' => ( is => 'rw', isa => 'Bool', default => 0 );
 
@@ -49,17 +48,10 @@ sub BUILD {
     }
 }
 
-sub _build_funcname {
-    my $self = shift;
-    my $funcname = $self->client->funcid_to_name($self->dbh, $self->funcid)
-        or die "Failed to lookup funcname of job $self";
-    return $funcname;
-}
-
 sub debug {
     my ($self, $msg) = @_;
     
-    $self->client->debug($msg, $self);
+    $self->handle->client->debug($msg, $self);
 }
 
 sub as_hashref {
@@ -70,27 +62,22 @@ sub as_hashref {
         $data{$col} = $self->$col if $self->can($col);
     }
 
-    return \%data;
+    \%data;
 }
 
 sub add_failure {
     my $job = shift;
-    my $msg = shift;
-    $msg = '' unless defined $msg;
+    my ($msg) = @_;
     
-    my $table_error = $job->handle->client->prefix . 'error';
-    if (my $len = $job->handle->client->error_length) {
-        $msg = substr($msg,0,$len);
-    }
-    my $sql = qq~INSERT INTO $table_error (error_time, jobid, message, funcid) VALUES (?, ?, ?, ?)~;
-    my $dbh = $job->dbh;
+    my $sql = q~INSERT INTO error (error_time, jobid, message, funcid) VALUES (?, ?, ?, ?)~;
+    my $dbh = $job->handle->dbh;
     my $sth = $dbh->prepare($sql);
-    $sth->execute(time(), $job->jobid, $msg, $job->funcid);
+    $sth->execute(time(), $job->jobid, $msg || '', $job->funcid);
 
     # and let's lazily clean some errors while we're here.
     my $maxage = $TheSchwartz::Moosified::T_ERRORS_MAX_AGE || (86400*7);
     my $dtime  = time() - $maxage;
-    $dbh->do(qq~DELETE FROM $table_error WHERE error_time < $dtime~);
+    $dbh->do(qq~DELETE FROM error WHERE error_time < $dtime~);
 
     return 1;
 }
@@ -104,54 +91,32 @@ sub completed {
         return 0;
     }
     $job->did_something(1);
-    return run_in_txn {
-        $job->set_exit_status(0);
-        $job->remove();
-    } $job->dbh;
+    $job->set_exit_status(0);
+    $job->remove();
 }
 
 sub remove {
-    my $job = shift;
+    my ($job) = @_;
     
+    my $dbh = $job->handle->dbh;
     my $jobid = $job->jobid;
-    my $table_job = $job->handle->client->prefix . 'job';
-    $job->dbh->do(qq~DELETE FROM $table_job WHERE jobid = $jobid~);
+    $dbh->do(qq~DELETE FROM job WHERE jobid = $jobid~);
 }
 
+sub exit_status { shift->handle->exit_status }
+sub failure_log { shift->handle->failure_log }
+sub failures    { shift->handle->failures    }
+
 sub set_exit_status {
     my $job = shift;
     my($exit) = @_;
     my $class = $job->funcname;
     my $secs = $class->keep_exit_status_for or return;
-
-    my $t = time();
-    my $jobid = $job->jobid;
-    my $funcid = $job->funcid;
-    my @status = ($exit, $t, $t + $secs);
-    my $dbh = $job->dbh;
-    my $table_exitstatus = $job->handle->client->prefix . 'exitstatus';
-    my $needs_update = 0;
-    {
-        my $sth = $dbh->prepare(qq{
-            INSERT INTO $table_exitstatus
-            (funcid, status, completion_time, delete_after, jobid)
-            SELECT ?, ?, ?, ?, ?
-            WHERE NOT EXISTS (
-                SELECT 1 FROM $table_exitstatus WHERE jobid = ?
-            )
-        });
-        $sth->execute($funcid, @status, $jobid, $jobid);
-        $needs_update = ($sth->rows == 0);
-    }
-    if ($needs_update) {
-        # only update if this status is newest
-        my $sth = $dbh->prepare(qq{
-            UPDATE $table_exitstatus
-            SET status=?, completion_time=?, delete_after=?
-            WHERE jobid = ? AND completion_time < ?
-        });
-        $sth->execute(@status, $jobid, $t);
-    }
+    
+    my $sql = q~INSERT INTO exitstatus (jobid, funcid, status, completion_time, delete_after) VALUES (?, ?, ?, ?, ?)~;
+    my $dbh = $job->handle->dbh;
+    my $sth = $dbh->prepare($sql);
+    $sth->execute( $job->jobid, $job->funcid, $exit, time(), time() + $secs );
 
     # and let's lazily clean some exitstatus while we're here.  but
     # rather than doing this query all the time, we do it 1/nth of the
@@ -159,8 +124,8 @@ sub set_exit_status {
     # default n is 10% of the time, doing 100 deletes.
     my $clean_thres = $TheSchwartz::Moosified::T_EXITSTATUS_CLEAN_THRES || 0.10;
     if (rand() < $clean_thres) {
-        my $unixtime = sql_for_unixtime($dbh);
-        $dbh->do(qq~DELETE FROM $table_exitstatus WHERE delete_after < $unixtime~);
+        my $unixtime = sql_for_unixtime();
+        $dbh->do(qq~DELETE FROM exitstatus WHERE delete_after < $unixtime~);
     }
 
     return 1;
@@ -198,29 +163,23 @@ sub _failed {
     $job->did_something(1);
     $job->debug("job failed: " . ($msg || "<no message>"));
 
-    run_in_txn {
-        ## Mark the failure in the error table.
-        $job->add_failure($msg);
+    ## Mark the failure in the error table.
+    $job->add_failure($msg);
 
-        if ($_retry) {
-            my $table_job = $job->handle->client->prefix . 'job';
-            my $class = $job->funcname;
-            my @bind;
-            my $sql = qq{UPDATE $table_job SET };
-            if (my $delay = $class->retry_delay($failures)) {
-                my $run_after = time() + $delay;
-                $job->run_after($run_after);
-                push @bind, $run_after;
-                $sql .= qq{run_after = ?, };
-            }
-            $sql .= q{grabbed_until = 0 WHERE jobid = ?};
-            push @bind, $job->jobid;
-            $job->dbh->do($sql, {}, @bind);
-        } else {
-            $job->set_exit_status($exit_status || 1);
-            $job->remove();
+    if ($_retry) {
+        my $class = $job->funcname;
+        my $sql = q~UPDATE job SET ~;
+        if (my $delay = $class->retry_delay($failures)) {
+            my $run_after = time() + $delay;
+            $job->run_after($run_after);
+            $sql .= qq~run_after = $run_after, ~;
         }
-    } $job->dbh;
+        $sql .= q~grabbed_until=0~;
+        $job->handle->dbh->do($sql);
+    } else {
+        $job->set_exit_status($exit_status || 1);
+        $job->remove();
+    }
 }
 
 sub replace_with {
@@ -233,22 +192,31 @@ sub replace_with {
     }
     # Note: we don't set 'did_something' here because completed does it down below.
 
+    ## The new jobs @jobs should be inserted into the same database as $job,
+    ## which they're replacing. So get a driver for the database that $job
+    ## belongs to.
+    my $handle = $job->handle;
+    my $client = $handle->client;
+    my $dbh    = $handle->dbh;
+
     $job->debug("replacing job with " . (scalar @jobs) . " other jobs");
 
-    ## The new jobs @jobs should be inserted into the same database as $job,
-    ## which they're replacing.
-    run_in_txn {
-        ## Mark the original job as completed successfully.
-        $job->completed;
+    ## XXX? TODO
+    ## Start a transaction.
 
-        ## Insert the new jobs.
-        $job->client->_try_insert($_, $job->dbh) for @jobs;
-    } $job->dbh;
+    ## Insert the new jobs.
+    for my $j (@jobs) {
+        $client->insert($j);
+    }
+
+    ## Mark the original job as completed successfully.
+    $job->completed;
 }
 
 sub set_as_current {
     my $job = shift;
-    $job->client->current_job($job);
+    my $client = $job->handle->client;
+    $client->current_job($job);
 }
 
 sub _cond_thaw {
